-- Daten für alle Charaktere
TaskManagerData = {};
TaskManagerData.tasks = {};
TaskManagerData.characters = {};

local TYPE_VALUES = {
    "Other",
    "Quest",
    "Garison",
    "Order Hall",
    "PVP",
    "Raid",
    "Dungeon",
}

local REPEAT_VALUES = {
    "Never",
    "Daily (3 am)",
    "Weekly (Wed., 3 am)",
}

local function getCharacterName(actCharacter)
    local name = actCharacter.name;
    local actRealm = GetRealmName();
    
    if (actRealm) then
        if (actCharacter.realm) then
            if (actRealm ~= actCharacter.realm) then
                name = name.."@"..actCharacter.realm;
            end
        end
    end
    
    return name;
end

function TaskManagerConfig_InitData()
    TaskManagerData.tasks = TaskManagerData.tasks or {};
    TaskManagerData.characters = TaskManagerData.characters or {};
    TaskManagerData.collapsed = TaskManagerData.collapsed or {};
    
    TaskManagerData.hideDone = TaskManagerData.hideDone or false;
    
    -- add/update act. character
    local guid = UnitGUID("player");
    
    TaskManagerData.characters[guid] = TaskManagerData.characters[guid] or {};
        
    local name = UnitName("player");
    local realm = GetRealmName();
    TaskManagerData.characters[guid].name = name;
    TaskManagerData.characters[guid].realm = realm;
    
    local race, raceEn = UnitRace("player");
    TaskManagerData.characters[guid].race = race;
    TaskManagerData.characters[guid].raceEn = raceEn;
    
    local localizedClass, englishClass, classIndex = UnitClass("player");
    TaskManagerData.characters[guid].localizedClass = localizedClass;
    TaskManagerData.characters[guid].englishClass = englishClass;
    TaskManagerData.characters[guid].classIndex = classIndex;
end

function TaskManagerConfig_OnLoad(self)
    TaskManagerConfigFramePortrait:SetTexture("Interface\\Addons\\TaskManager\\icons\\minimap_icon");
    TaskManagerConfigFramePortrait:SetTexCoord(0, 1, 0, 1);
    TaskManagerConfigFrameTitleText:SetText(EDIT);
    
    tinsert(UISpecialFrames, "TaskManagerConfigFrame");
    
    -- Twinks nur über Auswahl
    TaskManagerConfigFrameWho:SetEnabled(false);
    
    hooksecurefunc(TaskManagerConfigFrame, "Show", TaskManagerConfig_TaskManagerConfigFrame_Show);
    hooksecurefunc(TaskManagerConfigFrame, "Hide", TaskManagerConfig_TaskManagerConfigFrame_Hide);
end

local function findTaskById(id)
    for index, data in ipairs(TaskManagerData.tasks) do
        if (data.id == id) then
            return data, index;
        end
    end
    
    return nil, -1;
end

local function appendByCategory(nodes, data)
    nodes[data.category] = nodes[data.category] or {};    
    tinsert(nodes[data.category], data);
end

local function appendByLocation(nodes, data)
    nodes[data.location] = nodes[data.location] or {};
    tinsert(nodes[data.location], data);
end

local function appendByType(nodes, data)
    local typestring = TYPE_VALUES[data.type];
    nodes[typestring] = nodes[typestring] or {};
    tinsert(nodes[typestring], data);
end

local function appendData(nodes, data)    
    if (TaskManagerData.orderBy == 1) then -- order by category
        appendByCategory(nodes, data);
    elseif (TaskManagerData.orderBy == 2) then -- order by location
        appendByLocation(nodes, data);
    elseif (TaskManagerData.orderBy == 3) then -- order by type
        appendByType(nodes, data);
    else -- default
        appendByCategory(nodes, data);
    end
end

local function orderTasks(guid)
    local nodes = {};
    
    local hideDone = TaskManagerData.hideDone;
    
    if (TaskManagerConfigFrame:IsVisible()) then
        hideDone = false;
    end
    
    local done;
    
    for index, data in ipairs(TaskManagerData.tasks) do
        if (TaskManagerConfigFrame:IsVisible() or data.active) then -- active tasks only
            if (not guid or tContains(data.characters, guid)) then -- nur aktueller Twink / alle wenn nil
                done = tContains(data.done, guid);
            
                if ((done and not hideDone) or not done) then -- hide done
                    appendData(nodes, data);
                end
            end
        end
    end
    
    return nodes;
end

local function pairsByKeysAsc(t, f)
    local a = {}
    for n in pairs(t) do table.insert(a, n) end
    table.sort(a, function(a,b) return a<b end)
    local i = 0      -- iterator variable
    local iter = function ()   -- iterator function
        i = i + 1
        if a[i] == nil then return nil
        else return a[i], t[a[i]]
        end
    end
    return iter
end

function TaskManagerConfig_ToggleEnabled()
    local enabled = true;

    if (not TaskManagerData.selectedId or not findTaskById(TaskManagerData.selectedId)) then
        enabled = false;
    end
    
    if (TM_CharacterMenu and TM_CharacterMenu:IsVisible()) then
        UIDropDownMenu_Initialize(TM_CharacterMenu, TaskManagerConfig_InitCharacterMenu, "MENU");
    end
    
    TaskManagerConfigFrameDeleteButton:SetEnabled(enabled);
    TaskManagerConfigFrameCheckBoxActivate:SetEnabled(enabled);
    TaskManagerConfigFrameCategory:SetEnabled(enabled);
    TaskManagerConfigFrameChangeCategory:SetEnabled(enabled);
    TaskManagerConfigFrameTitle:SetEnabled(enabled);
    TaskManagerConfigFrameChangeWho:SetEnabled(enabled);
    TaskManagerConfigFrameWhat:SetEnabled(enabled);
    TaskManagerConfigFrameWhere:SetEnabled(enabled);
    TaskManagerConfigFrameTypeDropDownButton:SetEnabled(enabled);
    TaskManagerConfigFrameRepeatDropDownButton:SetEnabled(enabled);
    TaskManagerConfigFrameSendButton:SetEnabled(enabled);
    
    if (enabled) then
        local task = findTaskById(TaskManagerData.selectedId);
        
        TaskManagerConfigFrameCheckBoxActivate:SetChecked(task.active);
        TaskManagerConfigFrameCategory:SetText(task.category);
        TaskManagerConfigFrameTitle:SetText(task.title);
        
        TaskManagerConfig_UpdateCharacterInfo(task);
        
        TaskManagerConfigFrameWhat:SetText(task.description);
        TaskManagerConfigFrameWhere:SetText(task.location);
        UIDropDownMenu_SetSelectedValue(TaskManagerConfigFrameTypeDropDown, task.type);
        UIDropDownMenu_SetText(TaskManagerConfigFrameTypeDropDown, TYPE_VALUES[task.type]);     
        UIDropDownMenu_SetSelectedValue(TaskManagerConfigFrameRepeatDropDown, task.refreshRate);
        UIDropDownMenu_SetText(TaskManagerConfigFrameRepeatDropDown, REPEAT_VALUES[task.refreshRate]);
    else    
        TaskManagerConfigFrameCheckBoxActivate:SetChecked(false);
        TaskManagerConfigFrameCategory:SetText("");
        TaskManagerConfigFrameTitle:SetText("");
        TaskManagerConfigFrameWho:SetText("");
        TaskManagerConfigFrameWhat:SetText("");
        TaskManagerConfigFrameWhere:SetText("");
        UIDropDownMenu_SetText(TaskManagerConfigFrameTypeDropDown, "");
        UIDropDownMenu_SetText(TaskManagerConfigFrameRepeatDropDown, "");
    end
end

function TaskManagerConfig_UpdateCharacterInfo(task)        
    local characters = "";
    local actCharacter;
    for index, guid in ipairs(task.characters) do
        actCharacter = TaskManagerData.characters[guid];
    
        local classColor = RAID_CLASS_COLORS[actCharacter.englishClass];
        local classColorText = string.format("ff%.2x%.2x%.2x", classColor.r * 255, classColor.g * 255, classColor.b * 255);
        
        name = getCharacterName(actCharacter);
        
        characters = characters.."|c"..classColorText..name.." |r";
    end
    
    TaskManagerConfigFrameWho:SetText(characters);
end

function TaskManagerConfig_TaskManagerConfigFrame_Show()
    TaskManagerConfig_ToggleEnabled();
    TaskManager_TasksOnUpdate();
end

function TaskManagerConfig_TaskManagerConfigFrame_Hide()
    TaskManagerData.selectedId = nil;
    TaskManager_TasksOnUpdate();
end

function TaskManagerConfig_OnShow()
    TaskManagerConfig_ReinitSettings();
end

function TaskManagerConfig_ReinitSettings()
    TaskManager_TasksOnUpdate();
end

function TaskManagerConfigFrame_OnMouseDown(frame, button)
    if (button == "LeftButton") then
        TaskManagerConfigFrame:StartMoving();
    end
end

function TaskManagerConfigFrame_OnMouseUp(frame, button)
    TaskManagerConfigFrame:StopMovingOrSizing();
end

function TaskManagerConfig_GetOpenTasks(guid)
    local nodes = {};
    
    local done;
    for index, data in ipairs(TaskManagerData.tasks) do 
        if (not guid or tContains(data.characters, guid)) then -- nur aktueller Twink / alle wenn nil
            done = tContains(data.done, guid);
        
            if (not done) then
                tinsert(nodes, data);
            end
        end
    end
    
    return nodes;
end

function TaskManagerConfig_GetAllTasks(guid)
    local nodes = {};
    
    nodes = orderTasks(guid);

    local sorted = {};
    
    for category, cat_node in pairsByKeysAsc(nodes) do     
        local cat_task = {};
        cat_task.title = category;
        cat_task.isHeader = true;
        cat_task.countTasks = #cat_node;
        
        tinsert(sorted, cat_task);
            
        if (not tContains(TaskManagerData.collapsed, category)) then   
            for index, data in ipairs(cat_node) do
                tinsert(sorted, data);
            end
        end
    end
    
    
    return sorted;
end

function TaskManagerConfig_Send()
tm_debug("SEND");
end

function TaskManagerConfig_New()
    TaskManagerConfig_CreateTask(TM_NEW_TASK, TM_NEW_CATEGORY, "", 1, 1);
end

function TaskManagerConfig_CreateTask(title, category, description, refreshRate, type)
    local task = {};
    
    task.id = 1;
    
    while (findTaskById(task.id)) do
        task.id = task.id + 1;
    end
    
    task.active = true;
    task.title = title;
    task.category = category;
    
    task.characters = {};
    local guid = UnitGUID("player");
    tinsert(task.characters, guid);
    
    task.description = description;
    
    task.location = "";
    
    task.refreshRate = refreshRate; -- 1 = none, 2 = daily, 3 = weekly
    task.type = type; -- Quest, Garrison, ..., Other
    
    task.done = {};
        
    tinsert(TaskManagerData.tasks, task);
    
    TaskManagerData.selectedId = task.id;
    TaskManagerConfig_ToggleEnabled();
        
    TaskManager_TasksOnUpdate();
end

function TaskManagerConfig_ChangeTask()
    if (not TaskManagerData.selectedId or not findTaskById(TaskManagerData.selectedId)) then
       return;
    end
    
    local task = findTaskById(TaskManagerData.selectedId);
    
    task.active = TaskManagerConfigFrameCheckBoxActivate:GetChecked();
    task.title = TaskManagerConfigFrameTitle:GetText();
    task.category = TaskManagerConfigFrameCategory:GetText();
    
    task.description = TaskManagerConfigFrameWhat:GetText();
    
    task.location = TaskManagerConfigFrameWhere:GetText();
    
    task.refreshRate = UIDropDownMenu_GetSelectedValue(TaskManagerConfigFrameRepeatDropDown);
    task.type = UIDropDownMenu_GetSelectedValue(TaskManagerConfigFrameTypeDropDown);

    TaskManager_TasksOnUpdate();
end

function TaskManagerConfig_Delete()
    if (not TaskManagerData.selectedId or not findTaskById(TaskManagerData.selectedId)) then
       return;
    end
    
    local task, index = findTaskById(TaskManagerData.selectedId);
    
    tremove(TaskManagerData.tasks, index); 

    TaskManagerData.selectedId = nil;
    
    TaskManagerConfig_ToggleEnabled();
    TaskManager_TasksOnUpdate();
end

function TaskManagerConfig_ChangeCategory(self)
    if (not self.menu) then
        self.menu = CreateFrame("Frame", "TM_CategoryMenu", self, "UIDropDownMenuTemplate");
        UIDropDownMenu_Initialize(self.menu, TaskManagerConfig_InitCategoryMenu, "MENU");
    end
    
    ToggleDropDownMenu(1, nil, self.menu, self, 0, 0);
end

function TaskManagerConfig_InitCategoryMenu()

    local categories = {};

    for index, data in ipairs(TaskManagerData.tasks) do
        categories[data.category] = true;
    end
    
    for category, _ in pairsByKeysAsc(categories) do
        if (category ~= TM_NEW_CATEGORY) then
            local infoCategory = UIDropDownMenu_CreateInfo();
            infoCategory.text = category;
            infoCategory.value = category;
            infoCategory.isNotRadio = true;
            infoCategory.notCheckable = true;
            
            infoCategory.func = function(frame)
                                    TaskManagerConfigFrameCategory:SetText(category);
                                    TM_CategoryMenu:Hide();
                                end;
    
            UIDropDownMenu_AddButton(infoCategory);
        end
    end
    
    UIDropDownMenu_AddSeparator(UIDropDownMenu_CreateInfo(), UIDROPDOWNMENU_MENU_LEVEL);

    local infoCancel = UIDropDownMenu_CreateInfo();
    infoCancel.text = CLOSE;
    infoCancel.value = nil;
    infoCancel.isNotRadio = true;
    infoCancel.notCheckable = true;
    UIDropDownMenu_AddButton(infoCancel);
    
    PlaySound("igMainMenuOpen");
end

function TaskManagerConfig_ChangeWho(self)
    if (not self.menu) then
        self.menu = CreateFrame("Frame", "TM_CharacterMenu", self, "UIDropDownMenuTemplate");
        UIDropDownMenu_Initialize(self.menu, TaskManagerConfig_InitCharacterMenu, "MENU");
    end
    
    ToggleDropDownMenu(1, nil, self.menu, self, 0, 0);
end

function TaskManagerConfig_InitCharacterMenu()
    
    local task = findTaskById(TaskManagerData.selectedId);
    
    local isAccount = false;
    
    if (task) then
        isAccount = task.isAccount;
    end

    local infoAccount = UIDropDownMenu_CreateInfo();
    infoAccount.text = "Account";
    infoAccount.value = nil;
    infoAccount.isNotRadio = true;
    
    infoAccount.checked = isAccount;
    infoAccount.func = function(frame) TaskManager_ToggleCharacter("ACCOUNT"); end;
    UIDropDownMenu_AddButton(infoAccount);
    
    UIDropDownMenu_AddSeparator(UIDropDownMenu_CreateInfo(), UIDROPDOWNMENU_MENU_LEVEL);
    
    if (not isAccount) then

        local infoAll = UIDropDownMenu_CreateInfo();
        infoAll.text = ALL;
        infoAll.value = nil;
        infoAll.isNotRadio = true;
        infoAll.notCheckable = true;
        infoAll.func = function(frame) TaskManager_ToggleCharacter("ALL"); end;
        UIDropDownMenu_AddButton(infoAll);
    
        UIDropDownMenu_AddSeparator(UIDropDownMenu_CreateInfo(), UIDROPDOWNMENU_MENU_LEVEL);
    
        for guid, data in pairs(TaskManagerData.characters) do        
            local info = UIDropDownMenu_CreateInfo();
            
            local name = getCharacterName(data);  
            
            local classColor = RAID_CLASS_COLORS[data.englishClass];
            local classColorText = string.format("ff%.2x%.2x%.2x", classColor.r * 255, classColor.g * 255, classColor.b * 255);
            
            info.text = "|c"..classColorText..name..", "..data.race.." "..data.localizedClass.."|r";
            
            info.isNotRadio = true;
            info.keepShownOnClick = true;
            info.checked = TaskManager_ContainsCharacter(guid);
            info.func = function(frame) TaskManager_ToggleCharacter(guid); end;
       
            UIDropDownMenu_AddButton(info);
        end
        
        UIDropDownMenu_AddSeparator(UIDropDownMenu_CreateInfo(), UIDROPDOWNMENU_MENU_LEVEL);
    end

    local infoNone = UIDropDownMenu_CreateInfo();
    infoNone.text = "Deselect all";
    infoNone.value = nil;
    infoNone.isNotRadio = true;
    infoNone.notCheckable = true;
    infoNone.func = function(frame) TaskManager_DeselectAllCharacters(); end;
    UIDropDownMenu_AddButton(infoNone);
    
    UIDropDownMenu_AddSeparator(UIDropDownMenu_CreateInfo(), UIDROPDOWNMENU_MENU_LEVEL);

    local infoCancel = UIDropDownMenu_CreateInfo();
    infoCancel.text = CLOSE;
    infoCancel.value = nil;
    infoCancel.isNotRadio = true;
    infoCancel.notCheckable = true;
    UIDropDownMenu_AddButton(infoCancel);
    
    PlaySound("igMainMenuOpen");
end

function TaskManagerConfig_ChangeWhere(self)
    TaskManagerConfigFrameWhere:SetText(GetMinimapZoneText());
end

local function findCharIndexByGuid(task, guid)
    for index, foundguid in ipairs(task.characters) do
        if (foundguid == guid) then
            return index;
        end
    end
    
    return nil;
end

function TaskManager_ContainsCharacter(guid)        
    local task = findTaskById(TaskManagerData.selectedId);
    
    if (not task) then
        return false;
    end

    local index = findCharIndexByGuid(task, guid);
    
    return index and index > 0;

end

function TaskManager_DeselectAllCharacters()   
    local task = findTaskById(TaskManagerData.selectedId);
    
    task.isAccount = false;
    task.characters = {};
    
    TaskManagerConfig_UpdateCharacterInfo(task);
    TaskManager_TasksOnUpdate();
end

function TaskManager_ToggleCharacter(guid)   
 
    local task = findTaskById(TaskManagerData.selectedId);
    
    if (not task) then
        return;
    end
    
    if (guid == "ALL" or guid == "ACCOUNT") then -- all
        task.characters = {};
        for guid, data in pairs(TaskManagerData.characters) do
            tinsert(task.characters, guid);
        end
    else -- toggle selection
        local index = findCharIndexByGuid(task, guid);
        
        if (index) then
            tremove(task.characters, index);
        else
            tinsert(task.characters, guid);
        end
    end
    
    if (guid == "ACCOUNT") then -- all
        task.isAccount = not task.isAccount;
    end
    
    TaskManagerConfig_UpdateCharacterInfo(task);
    TaskManager_TasksOnUpdate();
end

function TaskManager_TypeDropDown_Initialize()
    for index, data in ipairs(TYPE_VALUES) do
        local info = UIDropDownMenu_CreateInfo();
        info.text = data;
        info.value = index;
        info.func = function()
            TaskManager_TypeSelect(index);
        end
        UIDropDownMenu_AddButton(info);
    end
end

function TaskManager_GetTypeString(id)
    return TYPE_VALUES[id];
end

function TaskManager_GetRepeatString(id)
    return REPEAT_VALUES[id];
end

function TaskManager_TypeSelect(index)
    UIDropDownMenu_SetText(TaskManagerConfigFrameTypeDropDown, TYPE_VALUES[index]);   
    UIDropDownMenu_SetSelectedValue(TaskManagerConfigFrameTypeDropDown, index);
        
    TaskManagerConfig_ReinitSettings();
    TaskManagerConfig_ChangeTask();
end

function TaskManager_RepeatDropDown_Initialize()
    for index, data in ipairs(REPEAT_VALUES) do
        local info = UIDropDownMenu_CreateInfo();
        info.text = data;
        info.value = index;
        info.func = function()
            TaskManager_RepeatSelect(index);
        end
        UIDropDownMenu_AddButton(info);
    end
end

function TaskManager_RepeatSelect(index)
    UIDropDownMenu_SetText(TaskManagerConfigFrameRepeatDropDown, REPEAT_VALUES[index]);   
    UIDropDownMenu_SetSelectedValue(TaskManagerConfigFrameRepeatDropDown, index); 
        
    TaskManagerConfig_ReinitSettings();
    TaskManagerConfig_ChangeTask();
end
